@@ -34,7 +34,7 @@ from globus_automate_client.cli.helpers import (
 )
 from globus_automate_client.cli.rich_helpers import (
     FlowListDisplayFields,
-    LogCompletionDetetector,
+    LogCompletionDetector,
     RequestRunner,
     RunEnumerateDisplayFields,
     RunListDisplayFields,
@@ -164,7 +164,7 @@ def flow_deploy(
     ),
     validate: bool = typer.Option(
         True,
-        help=("(EXPERIMENTAL) Perform rudimentary validation of the flow definition."),
+        help="(EXPERIMENTAL) Perform rudimentary validation of the flow definition.",
         case_sensitive=False,
         show_default=True,
     ),
@@ -523,7 +523,7 @@ def flow_display(
         flow_dict = process_input(flow_definition)
 
     if output_format in {ImageOutputFormat.json, ImageOutputFormat.yaml}:
-        rr.render_as_result(flow_dict)
+        rr.render_as_result(flow_dict, client=fc, status_code=result.result.http_status)
     else:
         output_format.visualize(flow_dict)
 
@@ -977,7 +977,7 @@ def flow_action_log(
         min=1,
         max=100,
     ),
-    marker: str = typer.Option(
+    marker: Optional[str] = typer.Option(
         None,
         "--marker",
         "-m",
@@ -1032,7 +1032,7 @@ def flow_action_log(
         verbose=verbose,
         watch=watch,
         fields=RunLogDisplayFields,
-        detetector=LogCompletionDetetector,
+        detector=LogCompletionDetector,
     )
     with live_content:
         if output_format in {
